/*
 * q_ingress.c - Ingress qdisc
 *
 * Written 2001 by Werner Almesberger
 * Copyright 2001 EPFL-ICA, Network Robots
 */


#include <stddef.h>
#include <assert.h>

#include "util.h"
#include "error.h"
#include "param.h"
#include "tree.h"
#include "tc.h"
#include "filter.h"
#include "qdisc_common.h"
#include "qdisc.h"


/* ------------------------------- Checking -------------------------------- */


static void ingress_check(QDISC *qdisc)
{
    CLASS *class;

    qdisc->number = 0xffff; /* ingress magic number */
    for (class = qdisc->classes; class; class = class->next) {
	check_childless(class);
	if (class->qdisc) error("ingress has no inner qdiscs");
	if (class->number == UNDEF_U32)
	    error("ingress does not auto-assign class numbers");
    }
    sort_classes(&qdisc->classes);
    check_filters(qdisc->filters);
}


/* ------------------------ Default classification ------------------------- */


/*
 * We cheat a little. The "real" ingress simply puts some undefined value
 * into skb->tc_index. Not nice.
 */


static void ingress_default_class(DATA *d,QDISC *qdisc)
{
    append_default(d,data_decision(dt_class,require_class(qdisc,0)));
}


/* -------------------------------- Dump tc -------------------------------- */


static void ingress_dump_tc(QDISC *qdisc)
{
    CLASS *class;

    tc_pragma(qdisc->params);
    tc_qdisc_add(qdisc);
    tc_nl();
    for (class = qdisc->classes; class; class = class->next)
	tc_pragma(class->params);
    dump_filters(qdisc->filters);
}


/* ------------------------------ Descriptors ------------------------------ */


static const PARAM_DSC *ingress_qdisc_opt[] = {
    &prm_pragma,	/* list */
    NULL
};

static const PARAM_DSC *ingress_class_opt[] = {
    &prm_pragma,	/* list */
    NULL
};

static PARAM_DEF ingress_qdisc = {
    .required = NULL,
    .optional = ingress_qdisc_opt,
};

static PARAM_DEF ingress_class = {
    .required = NULL,
    .optional = ingress_class_opt,
};

QDISC_DSC ingress_dsc = {
    .name = "ingress",
    .flags = QDISC_HAS_CLASSES | QDISC_HAS_FILTERS | QDISC_HAS_DEFAULT,
    .qdisc_param = &ingress_qdisc,
    .class_param = &ingress_class,
    .check = &ingress_check,
    .default_class = &ingress_default_class,
    .dump_tc = &ingress_dump_tc,
    .dump_ext = &generic_dump_ext,
};
